#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "parse.h"


strtable *
parse(char *filename)
{
  char line[1024];
  size_t allocated = 1024;
  FILE *stream;
  strtable *table = malloc(sizeof(strtable));
  if (!table)
    return NULL;
  table->n = 0;
  if (!(table->string = malloc(allocated * sizeof(char *))))
    {
      fprintf(stderr, "malloc error\n");
      free(table);
      return NULL;
    }
  if (!(table->nboccur = malloc(allocated * sizeof(unsigned long long))))
    {
      fprintf(stderr, "malloc error\n");
      free(table->string);
      free(table);
      return NULL;
    }
  if (!(stream = fopen(filename, "r")))
    {
      fprintf(stderr, "Error opening %s\n", filename);
      free(table->nboccur);
      free(table->string);
      free(table);
      return NULL;
    }
  while (fgets(line, 1024, stream))
    {
      unsigned i, j;
      if (table->n >= allocated)
	{
	  allocated *= 2;
	  if (!(table->string = realloc(table->string,
					allocated * sizeof(char *))))
	    {
	      fprintf(stderr, "realloc error\n");
	      free(table->nboccur);
	      free(table);
	      fclose(stream);
	      return NULL;
	    }
	  if (!(table->nboccur = realloc(table->nboccur,
					 allocated * sizeof(unsigned long long))))
	    {
	      fprintf(stderr, "realloc error\n");
	      free(table->string);
	      free(table);
	      fclose(stream);
	      return NULL;
	    }
	}
      for (i = 0; i < 1024 && line[i] != ';'; i++) ;
      if (i == 1024 || line[i] != ';')
	{
	  fprintf(stderr, "Parsing error, expecting "";""\n");
	  free(table->nboccur);
	  free(table->string);
	  free(table);
	  fclose(stream);
	  return NULL;
	}
      line[i] = 0;
      table->string[table->n] = strdup(line);
      j = ++i;
      for (; i < 1024 &&  line[i] >= '0' && line[i] <= '9'; i++) ;
      if (i == 1024 || line[i] != '\n')
	{
	  fprintf(stderr, "Parsing error, expecting end of line\n");
	  free(table->nboccur);
	  free(table->string);
	  free(table);
	  fclose(stream);
	  return NULL;
	}
      line[i] = 0;
      table->nboccur[table->n] = atoll(line + j);
      table->n++;
    }
  fclose(stream);
  return table;
}

void
freestrtable(strtable* table)
{
  for (size_t i = 0; i < table->n; i++)
    free(table->string[i]);
  free(table->string);
  free(table->nboccur);
  free(table);
}
